# Generated by Django 3.0.3 on 2020-06-22 14:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="NotificationGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="created",
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="modified",
                    ),
                ),
                (
                    "code",
                    models.CharField(
                        blank=True, max_length=256, unique=True, verbose_name="code"
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notification_group",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "notification group",
                "verbose_name_plural": "notification groups",
                "ordering": ["-created"],
            },
        )
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WebsocketNotificationsAppConfig(AppConfig):

    name = "websocket_notifications"
    verbose_name = _("Websocket notifications")



import hashlib
import random
import time
from typing import Dict, Type

from django.apps import apps
from django.contrib.auth import get_user_model

User = get_user_model()


def generate_random_code(seed: str) -> str:
    """Generates a random code using the given seed."""
    return hashlib.sha256(
        ("{}-{}-{}".format(seed, time.time(), random.randint(100, 999))).encode("utf-8")
    ).hexdigest()


def send_to_user(user: Type[User], payload: Dict) -> None:
    """Gets the notifications group and sends the payload."""
    NotificationGroup = apps.get_model("websocket_notifications.NotificationGroup")
    notification_group, _ = NotificationGroup.objects.get_or_create_for_user(user)
    notification_group.send(payload=payload)



from snitch.backends import AbstractBackend
from snitch.settings import ENABLED_SEND_NOTIFICATIONS

from websocket_notifications.helpers import send_to_user


class WebSocketNotificationBackend(AbstractBackend):
    """A backend class to send websocket notifications."""

    @staticmethod
    def _content_type_serializer(content_type):
        return f"{content_type.app_label}.{content_type.model}"

    def payload(self):
        """Creates the payload of the message."""
        return {
            "notification": self.notification.id if self.notification else None,
            "event": {
                "actor_content_type": self._content_type_serializer(
                    self.event.actor_content_type
                ),
                "actor_object_id": self.event.actor_object_id,
                "verb": self.event.verb,
                "trigger_content_type": self._content_type_serializer(
                    self.event.trigger_content_type
                ),
                "trigger_object_id": self.event.trigger_object_id,
                "target_content_type": self._content_type_serializer(
                    self.event.target_content_type
                ),
                "target_object_id": self.event.target_object_id,
            },
        }

    def send(self):
        """Send message."""
        if ENABLED_SEND_NOTIFICATIONS:
            send_to_user(self.user, self.payload())



import json

from channels.db import database_sync_to_async
from channels.generic.websocket import AsyncWebsocketConsumer

from websocket_notifications.models import NotificationGroup


class NotificationConsumer(AsyncWebsocketConsumer):
    """Consumer for sending notification to the client."""

    @database_sync_to_async
    def get_notification_group(self, code):
        return NotificationGroup.objects.filter(code=code).first()

    async def connect(self):
        """When a consumer is connected, we get the code from the URL, to join to
        the group."""
        self.code = self.scope["url_route"]["kwargs"]["code"]
        if await self.get_notification_group(self.code):
            await self.channel_layer.group_add(self.code, self.channel_name)
            await self.accept()

    async def disconnect(self, close_code):
        """On disconnect, exit from the group."""
        await self.channel_layer.group_discard(self.code, self.channel_name)

    async def notification_message(self, event):
        """When received a message from a notification."""
        payload = event["payload"]
        await self.send(text_data=json.dumps({"payload": payload}))



"""Django app made to handle notifications (like Django Snitch) using 
websockets and Django Channels.
"""
__version__ = "1.0.1"


default_app_config = "websocket_notifications.apps.WebsocketNotificationsAppConfig"



from django.contrib import admin

from websocket_notifications.models import NotificationGroup


@admin.register(NotificationGroup)
class NotificationGroupAdmin(admin.ModelAdmin):
    list_display = ["id", "user", "code", "created"]
    autocomplete_fields = ["user"]
    search_fields = ["code", "user__email"]
